package com.supermap.imb.collectordemo;

import java.util.Calendar;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

import com.supermap.data.Point2D;
import com.supermap.plugin.LocationManagePlugin.GPSData;
import com.tencent.tencentmap.lbssdk.TencentMapLBSApi;
import com.tencent.tencentmap.lbssdk.TencentMapLBSApiListener;
import com.tencent.tencentmap.lbssdk.TencentMapLBSApiResult;

public class LocationIMB {
	
	// 
	private int m_reqGeoType = TencentMapLBSApi.GEO_TYPE_WGS84;
	private int m_reqLevel   = 0;
	private int m_reqDelay   = 1;
	// 
	private TencentMapLBSApiResult m_locationIfo = null;
	
	//
	private LocListener       m_Listener = null;
	
	//Current position information for navigation and cruise
	private GPSData m_GPSData = null;
	private Point2D m_Point = new Point2D();
	private double mAccuracy = 0;
	private MainActivity m_MainActivity = null;
	
	//System azimuth for positioning and cruise
	private SensorManager mSensorManager;
	private Sensor mSensorOrientation;
	
	/**
	 * azimuth
	 */
	private float mAzimuth = 0;
	/**
	 *Record the last azimuth
	 */
	private float mProAzimuth = 0;
	
	
	/**
	 * Construct function
	 * @param context
	 */
	public LocationIMB(MainActivity activity) {
		m_Listener = new LocListener(m_reqGeoType, m_reqLevel, m_reqDelay);
		
		int req = TencentMapLBSApi.getInstance().requestLocationUpdate(activity.getApplicationContext(), m_Listener);
		TencentMapLBSApi.getInstance().setGPSUpdateInterval(1000);
		if(req == -2)
			System.out.println("Key error, please set the correct key in the mainfext file");
		
		m_MainActivity = activity;
		
		mSensorManager = (SensorManager)activity.getSystemService(Context.SENSOR_SERVICE);
		mSensorOrientation = mSensorManager.getDefaultSensor(Sensor.TYPE_ORIENTATION);
		startSensor();
	}

	/**
	 * Get the position information
	 * @return
	 */
	public TencentMapLBSApiResult getLocationInfo() {
		return m_locationIfo;
	}
	
	/**
	 * Get the position information
	 * @return
	 */
	public GPSData getGPSData(){
		return m_GPSData;
	}
	
	/**
	 * Get current position
	 * @return
	 */
	public Point2D getGPSPoint(){
		return m_Point;
	}
	
	/**
	 * Get the current positioning accuracy
	 * @return
	 */
	public double getAccuracy(){
		return mAccuracy;
	}
	
	/**
	 * Close the positioning listener and azimuth listener
	 * @return
	 */
	public void closeLocation(){
		TencentMapLBSApi.getInstance().removeLocationUpdate();
		stopSensor();
	}
	/**
	 * Location listening class
	 *
	 */
	private class LocListener extends TencentMapLBSApiListener {

		// Initializes listener
		public LocListener(int reqGeoType, int reqLevel, int reqDelay) {
			super(reqGeoType, reqLevel, reqDelay);
		}
		// Update location information
		@Override
		public void onLocationUpdate(TencentMapLBSApiResult locResult) {
			
			m_locationIfo = locResult;
			
			//Current position information for navigation and cruise
			m_GPSData = new GPSData();
			m_GPSData.dAltitude = locResult.Altitude;
			m_GPSData.dLongitude = locResult.Longitude;
			m_GPSData.dLatitude = locResult.Latitude;
			m_GPSData.dSpeed  = locResult.Speed;
			//m_GPSData.dBearing = locResult.Bearing;
			Calendar  ca = Calendar.getInstance();
			m_GPSData.nYear = ca.get(Calendar.YEAR);
			m_GPSData.nMonth = ca.get(Calendar.MONTH);
			m_GPSData.nDay = ca.get(Calendar.DATE);
			m_GPSData.nHour = ca.get(Calendar.HOUR);
			m_GPSData.nMinute = ca.get(Calendar.MINUTE);
			m_GPSData.nSecond = ca.get(Calendar.SECOND);
			
			mAccuracy = locResult.Accuracy;
			
			m_Point.setX(locResult.Longitude);
			m_Point.setY(locResult.Latitude);
			if(locResult.Longitude == 0 && locResult.Latitude == 0 //No positioning
			/*|| !m_MainActivity.getMap().getBounds().contains(m_Point)*/){
				System.out.println("Unable to locate the position.");
			}
			else{
				m_MainActivity.mPoint2D.setX(locResult.Longitude);
				m_MainActivity.mPoint2D.setY(locResult.Latitude);
				m_MainActivity.mAccuracy = locResult.Accuracy;
			}
			m_MainActivity.drawCircleOnDyn(m_MainActivity.mPoint2D, mAzimuth, m_MainActivity.mAccuracy);
//			m_MainActivity.showInfo("get a new location.");
		}
	}
	
	/**
	 * The direction listening for the change of mobile azimuth.
	 */
	private SensorEventListener mSensorEventListener = new SensorEventListener() {
		
		@Override
		public void onSensorChanged(SensorEvent event) {
			// TODO Auto-generated method stub
			if (event.sensor.getType() == Sensor.TYPE_ORIENTATION) {
				mAzimuth = event.values[0];
				
				if (Math.abs(mAzimuth-mProAzimuth)<3) {
					return;
				}
				mProAzimuth = mAzimuth;
				//Draws the current azimuth after the change
				//m_MainActivity.drawCircleOnDyn(m_MainActivity.mCurrentPnt2D,mAzimuth);
			}
		}
		
		@Override
		public void onAccuracyChanged(Sensor sensor, int accuracy) {
			// TODO Auto-generated method stub
			
		}
	};
	
	/**
	 * Get the current azimuth
	 */
	public float getAzimuth(){
		return mAzimuth;
	}
	
	/**
	 * Start to listen the change of azimuthal
	 */
	public void startSensor(){
		if (mSensorManager != null && mSensorOrientation != null) {
			mSensorManager.registerListener(mSensorEventListener, mSensorOrientation, SensorManager.SENSOR_DELAY_UI);
		}
	}

	/**
	 * Stop listening the change of azimuthal
	 */
	public void stopSensor(){
		if (mSensorManager != null && mSensorOrientation != null) {
			mSensorManager.unregisterListener(mSensorEventListener, mSensorOrientation);
		}
	}
}
