package com.supermap.imb.collectordemo;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.PopupWindow;
import android.widget.TextView;

import com.supermap.data.GeoStyle;
import com.supermap.data.Recordset;
import com.supermap.mapping.Layers;
import com.supermap.mapping.Map;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.collector.Collector;
import com.supermap.mapping.collector.CollectorElement;

public class SettingPopup extends PopupWindow implements OnClickListener {

	private MapControl        m_MapControl     = null;
	private Layers            m_Layers         = null;
	private Map               m_Map            = null;

	private LayoutInflater    m_LayoutInflater = null;
	private View              m_ContentView    = null;
	
	private TextView mTip = null;
	private MainActivity    m_MainActivity     = null;

	/**
	 * Collector class
	 */
	private Collector m_Collector = null;

	/**
	 * Line color
	 */
	private com.supermap.data.Color   m_LineColor = new com.supermap.data.Color(82,198,223);
	
	/**
	 * Region color
	 */
	private com.supermap.data.Color   m_FillForeColor = new com.supermap.data.Color(160,255,90);
	
	/**
	 * Line width
	 */
	private double                    m_LineWidth = 1.0f;
	
	public SettingPopup (MapControl mapControl, Collector collector, MainActivity activity){
		super(activity);
    	
		m_LayoutInflater = LayoutInflater.from(mapControl.getContext());
		m_MapControl     = mapControl;
		m_Collector      = collector;
		m_MainActivity   = activity;
		
		initView();
		
		//Response the return key
		setBackgroundDrawable(new ColorDrawable(Color.WHITE));
	}
	
	private void initView() {
		// TODO Auto-generated method stub
		
		m_ContentView = m_LayoutInflater.inflate(R.layout.settingcollector, null);
		setContentView(m_ContentView);
		
		

		((Button) m_ContentView.findViewById(R.id.btnLineStyle)).setOnClickListener(this);
		((Button) m_ContentView.findViewById(R.id.btnPolygonStyle)).setOnClickListener(this);
		((Button) m_ContentView.findViewById(R.id.btnLineWidth)).setOnClickListener(this);
		((Button) m_ContentView.findViewById(R.id.btnSave)).setOnClickListener(this);
		((Button) m_ContentView.findViewById(R.id.btnCBack)).setOnClickListener(this);
		
	}
	
	@Override
	public void onClick(View arg0) {
		// TODO Auto-generated method stub
		switch(arg0.getId()){
		//Line style
		case R.id.btnLineStyle:
			if(m_Collector != null){
				int[] colors = {0,90,160,190,220,245};
				int valueR = colors[(int)Math.round(Math.random()*(colors.length-1))];
				int valueB = colors[(int)Math.round(Math.random()*(colors.length-1))];
				m_LineColor = new com.supermap.data.Color(valueR, 110, valueB);

				String strR = String.valueOf(valueR);
				String strB = String.valueOf(valueB);
				((EditText) m_ContentView.findViewById(R.id.EditText4)).setText("R= " + strR + ",G=" + 110 + ",B=" + strB);
			}
			break;
		//Region style
		case R.id.btnPolygonStyle:
			if(m_Collector != null){
				int[] colors = {0,90,160,190,220,245};
				int valueR = colors[(int)Math.round(Math.random()*(colors.length-1))];
				int valueB = colors[(int)Math.round(Math.random()*(colors.length-1))];
				m_FillForeColor = new com.supermap.data.Color(valueR, 255, valueB);

				String strR = String.valueOf(valueR);
				String strB = String.valueOf(valueB);
				((EditText) m_ContentView.findViewById(R.id.EditText5)).setText("R=" + strR + ",G=255" + ",B=" + strB);
			}
			break;
		//Line width
		case R.id.btnLineWidth:
			if(m_Collector != null){
				double[] widths = {0.6,1.0,1.2,1.5,1.6,1.8,1.9};
				double width = widths[(int)Math.round(Math.random()*(widths.length-1))];
				m_LineWidth = width;
				
				String strWidth = String.valueOf(width);
				((EditText) m_ContentView.findViewById(R.id.EditText6)).setText(strWidth);
			}
			break;
		//Save
		case R.id.btnSave:
			if(m_Collector != null){
				
				//Set the display style
				GeoStyle geoStyle = m_Collector.getStyle();//new GeoStyle();
				//Line width
				geoStyle.setLineWidth(m_LineWidth);
				//Line color
				geoStyle.setLineColor(m_LineColor);
				//Region color
				geoStyle.setFillForeColor(m_FillForeColor);
				//Set style
				m_Collector.setStyle(geoStyle);
				
				//Add the name and description
				CollectorElement element = m_Collector.getElement();
				if(element != null){
					String strName = ((EditText) m_ContentView.findViewById(R.id.EditText2)).getText().toString();
					String strNotes = ((EditText) m_ContentView.findViewById(R.id.EditText3)).getText().toString();
					
					//Name
					element.setName(strName);
					//Description
					element.setNotes(strNotes);
				}
				
				//Hide the setting window
				this.dismiss();
			}
			break;
			
		//Back
		case R.id.btnCBack:
			this.dismiss();
			break;
		default:
			break;
		}
	}
	
	/**
	 * Clear result
	 */
	public void clear(){
	}
	
    /**
     * Show
     */
   public void show(){
		showAt(0, 0, m_MapControl.getWidth(), m_MapControl.getHeight());
	}
	
	private void showAt(int x,int y, int width, int height)
	{
		setWidth(width);
		setHeight(height);
		
		showAtLocation(m_MapControl.getRootView(), Gravity.LEFT|Gravity.CENTER, dp2px(x), dp2px(y));
	}
	
	public void dismiss(){
		super.dismiss();
	}

	public int dp2px (int dp) 
	{
		int density = (int) (dp*m_MapControl.getContext().getResources().getDisplayMetrics().density);
		return density;
	}
	
}